"""
Elektron MCP (MIDI Control Protocol) Module

This module provides Python classes for working with Elektron synths using MIDI.
It uses Pydantic for data validation and JSON serialization.
"""

__version__ = "0.1.0"
